@@ -709,9 +709,10 @@ static int impl_node_process(void *object)
 		size = SPA_MIN(bd->maxsize - offs, bd->chunk->size);
 		maxsize = SPA_MIN(maxsize, size);
 
-		spa_log_trace_fp(this->log, "%p: mix input %d %p->%p %d %d %d:%d/%d", this,
+		spa_log_trace_fp(this->log, "%p: mix input %d %p->%p %d %d %d:%d/%d %u", this,
 				i, inio, outio, inio->status, inio->buffer_id,
-				offs, size, (int)sizeof(float));
+				offs, size, (int)sizeof(float),
+				bd->chunk->flags);
 
 		if (!SPA_FLAG_IS_SET(bd->chunk->flags, SPA_CHUNK_FLAG_EMPTY)) {
 			datas[n_buffers] = SPA_PTROFF(bd->data, offs, void);
