@@ -1,27 +1,7 @@
-/* PipeWire
- *
- * Copyright © 2021 Wim Taymans <wim.taymans@gmail.com>
- * Copyright © 2021 Sanchayan Maity <sanchayan@asymptotic.io>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2021 Wim Taymans <wim.taymans@gmail.com> */
+/* SPDX-FileCopyrightText: Copyright © 2021 Sanchayan Maity <sanchayan@asymptotic.io> */
+/* SPDX-License-Identifier: MIT */
 
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -45,9 +25,8 @@ struct module_pipesink_data {
 	struct spa_hook mod_listener;
 	struct pw_impl_module *mod;
 
+	struct pw_properties *global_props;
 	struct pw_properties *capture_props;
-	struct spa_audio_info_raw info;
-	char *filename;
 };
 
 static void module_destroy(void *data)
@@ -69,7 +48,6 @@ static int module_pipe_sink_load(struct module *module)
 	FILE *f;
 	char *args;
 	size_t size;
-	uint32_t i;
 
 	pw_properties_setf(data->capture_props, "pulse.module.id",
 			"%u", module->index);
@@ -78,23 +56,7 @@ static int module_pipe_sink_load(struct module *module)
 		return -errno;
 
 	fprintf(f, "{");
-	fprintf(f, " \"tunnel.mode\" = \"sink\" ");
-	if (data->filename != NULL)
-		fprintf(f, " \"pipe.filename\": \"%s\"", data->filename);
-	if (data->info.format != 0)
-		fprintf(f, " \"audio.format\": \"%s\"", format_id2name(data->info.format));
-	if (data->info.rate != 0)
-		fprintf(f, " \"audio.rate\": %u,", data->info.rate);
-	if (data->info.channels != 0) {
-		fprintf(f, " \"audio.channels\": %u,", data->info.channels);
-		if (!(data->info.flags & SPA_AUDIO_FLAG_UNPOSITIONED)) {
-			fprintf(f, " \"audio.position\": [ ");
-			for (i = 0; i < data->info.channels; i++)
-				fprintf(f, "%s\"%s\"", i == 0 ? "" : ",",
-					channel_id2name(data->info.position[i]));
-			fprintf(f, " ],");
-		}
-	}
+	pw_properties_serialize_dict(f, &data->global_props->dict, 0);
 	fprintf(f, " \"stream.props\": {");
 	pw_properties_serialize_dict(f, &data->capture_props->dict, 0);
 	fprintf(f, " } }");
@@ -125,7 +87,7 @@ static int module_pipe_sink_unload(struct module *module)
 		d->mod = NULL;
 	}
 	pw_properties_free(d->capture_props);
-	free(d->filename);
+	pw_properties_free(d->global_props);
 	return 0;
 }
 
@@ -146,30 +108,30 @@ static int module_pipe_sink_prepare(struct module * const module)
 {
 	struct module_pipesink_data * const d = module->user_data;
 	struct pw_properties * const props = module->props;
-	struct pw_properties *capture_props = NULL;
+	struct pw_properties *global_props = NULL, *capture_props = NULL;
 	struct spa_audio_info_raw info = { 0 };
 	const char *str;
-	char *filename = NULL;
 	int res = 0;
 
 	PW_LOG_TOPIC_INIT(mod_topic);
 
+	global_props = pw_properties_new(NULL, NULL);
 	capture_props = pw_properties_new(NULL, NULL);
-	if (!capture_props) {
+	if (!global_props || !capture_props) {
 		res = -EINVAL;
 		goto out;
 	}
 
-	if (module_args_to_audioinfo(module->impl, props, &info) < 0) {
-		res = -EINVAL;
-		goto out;
-	}
+	pw_properties_set(global_props, "tunnel.mode", "sink");
 
 	info.format = SPA_AUDIO_FORMAT_S16;
-	if ((str = pw_properties_get(props, "format")) != NULL) {
-		info.format = format_paname2id(str, strlen(str));
-		pw_properties_set(props, "format", NULL);
+	if (module_args_to_audioinfo_keys(module->impl, props,
+			"format", "rate", "channels", "channel_map", &info) < 0) {
+		res = -EINVAL;
+		goto out;
 	}
+	audioinfo_to_properties(&info, global_props);
+
 	if ((str = pw_properties_get(props, "sink_name")) != NULL) {
 		pw_properties_set(capture_props, PW_KEY_NODE_NAME, str);
 		pw_properties_set(props, "sink_name", NULL);
@@ -178,7 +140,7 @@ static int module_pipe_sink_prepare(struct module * const module)
 		module_args_add_props(capture_props, str);
 
 	if ((str = pw_properties_get(props, "file")) != NULL) {
-		filename = strdup(str);
+		pw_properties_set(global_props, "pipe.filename", str);
 		pw_properties_set(props, "file", NULL);
 	}
 	if ((str = pw_properties_get(capture_props, PW_KEY_DEVICE_ICON_NAME)) == NULL)
@@ -189,14 +151,13 @@ static int module_pipe_sink_prepare(struct module * const module)
 				"fifo_output");
 
 	d->module = module;
+	d->global_props = global_props;
 	d->capture_props = capture_props;
-	d->info = info;
-	d->filename = filename;
 
 	return 0;
 out:
+	pw_properties_free(global_props);
 	pw_properties_free(capture_props);
-	free(filename);
 	return res;
 }
 
