@@ -62,6 +62,10 @@
 PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
 #define PW_LOG_TOPIC_DEFAULT mod_topic
 
+/* libcanberra is not thread safe when doing ca_context_create()
+ * and so we need a global lock */
+static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
+
 struct impl {
 	struct pw_context *context;
 	struct pw_thread_loop *thread_loop;
@@ -83,6 +87,7 @@ static int play_sample(struct impl *impl)
 	ca_context *ca;
 	int res;
 
+	pthread_mutex_lock(&lock);
 	if (impl->properties)
 		sample = pw_properties_get(impl->properties, "sample.name");
 	if (sample == NULL)
@@ -113,6 +118,7 @@ static int play_sample(struct impl *impl)
 exit_destroy:
 	ca_context_destroy(ca);
 exit:
+	pthread_mutex_unlock(&lock);
 	return res;
 }
 
