@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2019 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2019 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #ifndef PIPEWIRE_KEYS_H
 #define PIPEWIRE_KEYS_H
@@ -72,7 +52,8 @@ extern "C" {
 #define PW_KEY_OBJECT_LINGER		"object.linger"		/**< the object lives on even after the client
 								  *  that created it has been destroyed */
 #define PW_KEY_OBJECT_REGISTER		"object.register"	/**< If the object should be registered. */
-
+#define PW_KEY_OBJECT_EXPORT		"object.export"		/**< If the object should be exported,
+								  *  since 0.3.72 */
 
 /* config */
 #define PW_KEY_CONFIG_PREFIX		"config.prefix"		/**< a config prefix directory */
@@ -169,7 +150,8 @@ extern "C" {
 #define PW_KEY_NODE_LOCK_RATE		"node.lock-rate"	/**< don't change rate when this node
 								  *  is active */
 #define PW_KEY_NODE_FORCE_RATE		"node.force-rate"	/**< force a rate while the node is
-								  *  active */
+								  *  active. A value of 0 takes the denominator
+								  *  of node.rate */
 
 #define PW_KEY_NODE_DONT_RECONNECT	"node.dont-reconnect"	/**< don't reconnect this node. The node is
 								  *  initially linked to target.object or the
@@ -215,6 +197,8 @@ extern "C" {
 #define PW_KEY_PORT_CACHE_PARAMS	"port.cache-params"	/**< cache the node port params */
 #define PW_KEY_PORT_EXTRA		"port.extra"		/**< api specific extra port info, API name
 								  *  should be prefixed. "jack:flags:56" */
+#define PW_KEY_PORT_PASSIVE		"port.passive"		/**< the ports wants passive links, since 0.3.67 */
+#define PW_KEY_PORT_IGNORE_LATENCY	"port.ignore-latency"	/**< latency ignored by peers, since 0.3.71 */
 
 /** link properties */
 #define PW_KEY_LINK_ID			"link.id"		/**< a link id */
@@ -292,7 +276,10 @@ extern "C" {
 #define PW_KEY_STREAM_LATENCY_MAX	"stream.latency.max"	/**< The maximum latency of the stream */
 #define PW_KEY_STREAM_MONITOR		"stream.monitor"	/**< Indicates that the stream is monitoring
 								  *  and might select a less accurate but faster
-								  *  conversion algorithm. */
+								  *  conversion algorithm. Monitor streams are also
+								  *  ignored when calculating the latency of their peer
+								  *  ports (since 0.3.71).
+								  */
 #define PW_KEY_STREAM_DONT_REMIX	"stream.dont-remix"	/**< don't remix channels */
 #define PW_KEY_STREAM_CAPTURE_SINK	"stream.capture.sink"	/**< Try to capture the sink output instead of
 								  *  source output */
@@ -342,8 +329,13 @@ extern "C" {
 								  * and object name or object.serial */
 
 #ifndef PW_REMOVE_DEPRECATED
-#define PW_KEY_PRIORITY_MASTER		PW_DEPRECATED("priority.master")	/**< deprecated, use priority.driver */
-#define PW_KEY_NODE_TARGET		PW_DEPRECATED("node.target")		/**< deprecated since 0.3.64, use target.object. */
+# ifdef PW_ENABLE_DEPRECATED
+#  define PW_KEY_PRIORITY_MASTER	"priority.master"	/**< deprecated, use priority.driver */
+#  define PW_KEY_NODE_TARGET		"node.target"		/**< deprecated since 0.3.64, use target.object. */
+# else
+#  define PW_KEY_PRIORITY_MASTER	PW_DEPRECATED("priority.master")
+#  define PW_KEY_NODE_TARGET		PW_DEPRECATED("node.target")
+# endif /* PW_ENABLE_DEPRECATED */
 #endif /* PW_REMOVE_DEPRECATED */
 
 /** \}
